<?php
/*
 * The file hwm_universal_interface_config.php contains the configuration for
 * the HWM's universal interface, which is used for data exchange between a
 * HWM based application (e.g. ProcMan) and external applications. For the data
 * exchange the external applications must provide a Web-interface supporting
 * either http or https.
 *
 * If it is required and properly configured HWM can inform the external
 * applications, when the status of an activity and/or a process has been
 * changed. The universal interface can also be used for customer specific
 * issues, when for example valid HWM process parameter values have to be
 * requested from an external application. In this case the universal
 * interface has to be executed by explicit calling the HWM API function
 * hwm_universal_interface_exec().
 *
 * For each partial communication a section with a unique id has to be
 * configured in the 'communication' section of the
 * $hwm_universal_interface_config structure (see sections with ids 'com1' and
 * 'com2' in the example below). Each such a communication section can
 * contain a 'connection' and a 'callback' section.
 *
 * The 'connection' section specifies parameters required for the connection
 * with the external system and parameters containing data sent to the 
 * external system. The names of the parameters can be specified in three
 * different forms. For example the following specifications are equivalent:
 *   'url' => ...,
 *   'curlopt_url' => ...,
 *   CURLOPT_URL => ...,
 * Thereby the string forms are case insensitive. In the example below only
 * the most common parameters are used in the 'connection' sections. For all
 * possible parameters see http://php.net/manual/de/function.curl-setopt.php.
 *
 * In the 'callback' section the names of request and response callback
 * functions can be specified. The prototype of the request callback function
 * must be:
 *     mixed function(array &$connection, mixed $data, string &$errMsg)
 * Initially the $connection parameter contains what is specified in the
 * 'connection' section. The connection information can be extended in the
 * request callback function. Alternatively the 'connection' section can
 * be omitted in the configuration and be set only in the request callback
 * function.
 * The $data parameter contains the data passed in the call of
 * hwm_universal_interface_exec() function and contains data which shall be
 * sent in the request to the external system. The request callback function
 * can extract the data from this parameter and put it in the connection 
 * structure in a form in which they are expected for the data exchange.
 * If the request callback call fails for some reason the error message has
 * to be returned as a string in the $errMsg parameter.
 * If the request callback function call has been successful the function
 * returns true. If it fails it returns false and sets the error message in
 * the $errMsg parameter. Anything else than true or false returned by the
 * function is handled as an abort of the data exchange. On abort there will
 * be nothing send to or received from the external system for the partial
 * communication. This allows to control in the request callback function,
 * whether the data exchange shall take place or not, depending on the $data
 * parameter content.
 * The prototype of the response callback function must be:
 *     bool function(mixed &$response, string &$errMsg)
 * In the parameter $response the raw response got from the external system
 * is passed to the response callback function. The callback function can
 * parse it and transform it in something else (e.g. an array). 
 * If the response callback function call has been successful the function
 * returns true. If it fails it returns false and sets the error message in
 * the $errMsg parameter.
 *
 * Anything of 'connection', 'request' and/or 'response' can be omitted from
 * the configuration of the partial communication configuration. If 'connection'
 * is omitted the connection structure has to be set by the request callback
 * function. If 'request' is omitted only what is specified in 'connection' is
 * used in the request. If 'response' is omitted only the raw response got
 * from the external system is handled as a result.
 *
 * In the section 'event' is specified which defined communications, specified
 * by communication ids, shall be used when the activities and/or processes
 * change their status. If none are specified no information is being sent
 * to external systems if the status has been changed.
 * On status change the data passed to the request callback function have
 * the following structure:
 *   array(
 *       'status' => (string), // new status
 *       'general' => array(...), // see API f. hwm_get_process_info()
 *       'additional' => array(...), // see API f.
 *                                   // hwm_get_process_application_info()
 *       [ 'activity' => (int), ] // activity id only on activity status change
 *   )
 * The response callback function for status change events can maximally check,
 * whether the data exchange was successful or not, based on the response got
 * from the external system. Anything returned by the function in the
 * response parameter is ignored in this case.
 */



define('SNOW_URL', 'https://hor-114.horizont-it.de:11443/snow_simulator.php');
define('SNOW_AUTH', 'dXNlcjpwYXNzd29yZAo=');

define('SNOW_TICKET_OPEN_ACTIONS',
       array(
           // action => stage
           'IWS_NEW_BAS_UPD' => 'INT',
           'IWS_NEW_UAT_DATE' => 'UAT',
           'IWS_NEW_PRO_DATE' => 'PRO',
           'IWS_CHG_BAS_UPD' => 'INT',
           'IWS_CHG_UAT_DATE' => 'UAT',
           'IWS_CHG_PRO_DATE' => 'PRO',
           'IWS_DEL_BAS_UPD' => 'INT',
           'IWS_DEL_UAT_DATE' => 'UAT',
           'IWS_DEL_PRO_DATE' => 'PRO',
           'JCL_NEW_APPROVE' => 'INT',
           'JCL_NEW_UAT_DATE' => 'UAT',
           'JCL_NEW_PRO_DATE' => 'PRO',
       )
);
define('SNOW_TICKET_CONFIRM_ACTIONS',
       array(
           // action => stage
           'IWS_NEW_INT_CNF' => 'INT',
           'IWS_NEW_UAT_CNF' => 'UAT',
           'IWS_NEW_PRO_CNF' => 'PRO',
           'IWS_CHG_INT_CNF' => 'INT',
           'IWS_CHG_UAT_CNF' => 'UAT',
           'IWS_CHG_PRO_CNF' => 'PRO',
           'IWS_DEL_INT_CNF' => 'INT',
           'IWS_DEL_UAT_CNF' => 'UAT',
           'IWS_DEL_PRO_CNF' => 'PRO',
           'JCL_NEW_INT_CNF' => 'INT',
           'JCL_NEW_UAT_CNF' => 'UAT',
           'JCL_NEW_PRO_CNF' => 'PRO',
       )
);
define('SNOW_TICKET_CLOSE_ACTIONS',
       array(
           // action => stage
           'IWS_NEW_INT_UPD' => 'INT',
           'IWS_NEW_UAT_UPD' => 'UAT',
           'IWS_NEW_PRO_UPD' => 'PRO',
           'IWS_CHG_INT_UPD' => 'INT',
           'IWS_CHG_UAT_UPD' => 'UAT',
           'IWS_CHG_PRO_UPD' => 'PRO',
           'IWS_DEL_INT_UPD' => 'INT',
           'IWS_DEL_UAT_UPD' => 'UAT',
           'IWS_DEL_PRO_UPD' => 'PRO',
           'JCL_NEW_INT_UPD' => 'INT',
           'JCL_NEW_UAT_UPD' => 'UAT',
           'JCL_NEW_PRO_UPD' => 'PRO',
       )
);

define('SNOW_LOG', true);
define('SNOW_LOG_FILE', $hwi_config['log_dir'].'/snow-'.date('Y-m-d').'.log');



$hwm_universal_interface_config = array(
    'communication' => array(
        'snow' => array(
            'connection' => array(
                'url' => SNOW_URL,
                'ssl_verifypeer' => false,
                //'cainfo' => 'c:/HORIZONT/var/ssl/horizont.crt',
                'postfields' => array(),
                'httpheader' => array(),
            ),
            'callback' => array(
                'request' => 'snow_request',
                'response' => 'snow_response',
            ),
        ),
        'com1' => array(
            'connection' => array(
                'url' => 'https://extsys1.my_company.com:21443/response.php',
                'ssl_verifypeer' => true,
                'cainfo' => 'c:/HORIZONT/var/ssl/horizont.crt',
                'postfields' => array('param1' => 'val1', 'param2' => 'val2'),
                'httpheader' => array('some-header:value', 'header2:val2'),
            ),
            'callback' => array(
                'request' => 'request1',
                'response' => 'response1',
            ),
        ),
        'com2' => array(
            'connection' => array(
                'url' => 'http://extsys2.my_company.com:21080/response.php',
                'postfields' => array('param1' => 'val1', 'param2' => 'val2'),
                'httpheader' => array('some-header:value'),
            ),
            'callback' => array(
                'request' => 'request2',
                'response' => 'response2',
            ),
        ),
        'ext_cmd' => array(
            'connection' => array(
                'url' => 'https://extsys3.my_company.com:10443/hwm_executor.php',
                'ssl_verifypeer' => true,
                'cainfo' => 'c:/HORIZONT/var/ssl/horizont.crt',
                'timeout' => 30,
                'postfields' => array(),
                'httpheader' => array(),
            ),
            'callback' => array(
                'request' => 'ext_cmd_request',
                'response' => 'ext_cmd_response',
            ),
        ),
        // Example for Windows/Unix script handovers using the Universal
        // Object Module:
        'script_dispatcher' => array(
            'connection' => array(
                'url' => 'https://dispatchsys.my_company.com:15443/hwm_executor.php',
                'ssl_verifypeer' => true,
                'cainfo' => 'c:/HORIZONT/var/ssl/horizont.crt',
                'postfields' => array(),
                'httpheader' => array(),
            ),
            'callback' => array(
                'request' => 'script_dispatcher_request',
                'response' => 'script_dispatcher_response',
            ),
        ),
        'script_writer' => array(
            'connection' => array(
                // url and other params set in the request callback (see below)
                'timeout' => 30,
                'postfields' => array(),
                'httpheader' => array(),
            ),
            'callback' => array(
                'request' => 'script_writer_request',
                'response' => 'script_writer_response',
            ),
        ),
        // Example of a configuration for HWM REST API
        'rest_api' => array(
            'connection' => array(
                'url' => 'https://hor-114.horizont-it.de:11443/hwm_rest_api.php',
                'ssl_verifypeer' => false,
                // 'cainfo' => 'c:/HORIZONT/var/ssl/horizont.crt',
                'timeout' => 60,
                'postfields' => array(),
                'httpheader' => array(),
            ),
            'callback' => array(
                'request' => 'rest_api_request',
                'response' => 'rest_api_response',
            ),
        ),
        // Example of a configuration for autoexecution of activities
        'autoexec_activity' => array(
            'connection' => array(
                'url' => 'https://hor-114.horizont-it.de:11443/hwm_autoexec_frame.php',
                'ssl_verifypeer' => false,
                // 'cainfo' => 'c:/HORIZONT/var/ssl/horizont.crt',
                'timeout' => 3600,
                'postfields' => array(),
                'httpheader' => array(
                    'Content-Type: application/json;charset=UTF-8',
                    'Authorization: Basic cHJvY21hbjpVS2MxdVJ4QWJDalZNelEy',
                ),
            ),
            'callback' => array(
                'request' => 'autoexec_activity_request',
                'response' => 'autoexec_activity_response',
            ),
        ),
        // Example of a configuration for the autoexecution handler
        'autoexec_handler' => array(
            'connection' => array(
                'url' => 'https://hor-114.horizont-it.de:11443/hwm_autoexec_handler.php',
                'ssl_verifypeer' => false, 
                // 'cainfo' => 'c:/HORIZONT/var/ssl/horizont.crt',
                'timeout' => 3600,
                'postfields' => array(),
                'httpheader' => array(
                    'Content-Type: application/json;charset=UTF-8',
                    'Authorization: Basic cHJvY21hbjpVS2MxdVJ4QWJDalZNelEy',
                ),
            ),
            'callback' => array(
                'request' => 'autoexec_handler_request',
                'response' => 'autoexec_handler_response',
            ),
        ),
        // Example of a configuration for the autoexecution handler
        'file_sweeper' => array(
            'connection' => array(
                'url' => 'https://hor-114.horizont-it.de:11443/hwm_file_sweeper.php',
                'ssl_verifypeer' => false, 
                // 'ssl_verifyhost' => 0,
                // 'cainfo' => 'c:/HORIZONT/var/ssl/horizont.crt',
                'timeout' => 3600,
                'postfields' => array(),
                'httpheader' => array(
                    'Content-Type: application/json;charset=UTF-8',
                    'Authorization: Basic cHJvY21hbjpVS2MxdVJ4QWJDalZNelEy',
                ),
            ),
            'callback' => array(
                'request' => 'file_sweeper_request',
                'response' => 'file_sweeper_response',
            ),
        ),
        // Example of a configuration for the Control-M module
        'ctm' => array(
            'connection' => array(
                'url' => 'https://ctm.my_company.com:8444/automation-api',
                'ssl_verifypeer' => true,
                'cainfo' => 'c:/HORIZONT/var/ssl/horizont.crt',
                'timeout' => 120,
                'postfields' => array(),
                'httpheader' => array(),
            ),
            'callback' => array(
                'request' => 'ctm_request',
                'response' => 'ctm_response',
            ),
        ),
    ),

    'event' => array(
        // 'on_activity_status_changed' => 'snow',      /* <= uncomment to activate snow interaction */  
        //'on_process_status_changed' => array('com1', 'com2'),
    ),
);



function snow_request(&$connection, $data, &$errMsg)
{
    if (!isset($data['status'])
        || !isset($data['activity']['process'])
        || !isset($data['activity']['action']))
        return 0; // abort - do not contact SNOW
    $process = $data['activity']['process'];
    $action = $data['activity']['action'];
    if (isset(SNOW_TICKET_OPEN_ACTIONS[$action]))
    {
        $type = 'OPEN';
        $stage = SNOW_TICKET_OPEN_ACTIONS[$action];
    }
    else if (isset(SNOW_TICKET_CONFIRM_ACTIONS[$action]))
    {
        $type = 'CONFIRM';
        $stage = SNOW_TICKET_CONFIRM_ACTIONS[$action];
    }
    else if (isset(SNOW_TICKET_CLOSE_ACTIONS[$action]))
    {
        $type = 'CLOSE';
        $stage = SNOW_TICKET_CLOSE_ACTIONS[$action];
    }
    else
        return 0; // abort - do not contact SNOW
    $status = $data['status'];
    if (($status !== 'Complete' || $type === 'CONFIRM')
        && ($status !== 'Rejected' || $type === 'OPEN'))
        return 0; // abort - do not contact SNOW
    if ($type === 'CONFIRM')
        $type = 'CLOSE';
    if ($type === 'OPEN')
    {
        snow_context_data(array(
            'process' => $process,
            'param' => 'SNOW_ID_'.$stage,
        ));
        $memo = snow_get_process_memo($process);
        if ($memo === false)
        {
            $errMsg = hwm_get_last_error();
            return false;
        }
        $app = isset($data['additional']['BUSINESS_APP'])
               ? $data['additional']['BUSINESS_APP'] : '';
        $proj = isset($data['additional']['PROJECT_ID'])
                ? $data['additional']['PROJECT_ID'] : '';
        $date = isset($data['additional'][$stage.'_DATE'])
                ? $data['additional'][$stage.'_DATE'] : '';
        $date = hwm_db_date_adjust($date);
        if ($date === false)
            $date = '';
        else if (strlen($date) === 8)
            $date = substr($date, 6, 2).'-'.substr($date, 4, 2).'-'
                    .substr($date, 0, 4);
        $descr = isset($data['general']['process_description'])
                 ? $data['general']['process_description'] : '';
    }
    else
    {
        $param = 'SNOW_ID_'.$stage;
        if (!isset($data['additional'][$param]))
        {
            $errMsg = 'Missing SNOW ticket in the parameter \''.$param
                      .'\' of the process '.$process.'!';
            return false;
        }
        $ticket = $data['additional'][$param];
    }

    $connection['httpheader'] = array(
        'Content-Type: application/json;charset=UTF-8',
        'Authorization: Basic '.SNOW_AUTH,
    );
    $request = array(
        'type' => $type,
        'process' => $process,
        'action' => $action,
        'stage' => $stage,
        //'data' => $data,
    );
    if ($type === 'OPEN')
    {
        $request['sysparm_quantity'] = '1';
        $request['variables'] = array(
            'environment' => $stage,
            'application' => $app,
            'release_date' => $date,
            'description' => $memo,
            'project' => $proj,
            'title' => $descr,
        );
        $request['sysparm_item_guid'] = '76377ad5832c0214b119fb526daad30e';
        $request['get_portal_messages'] = 'true';
        $request['sysparm_no_validation'] = 'true';
        $request['engagement_channel'] = 'sp';
        $request['referrer'] = null;
    }
    else
        $request['ticket'] = $ticket;
    $connection['postfields'] = json_encode($request);
    snow_log('request', $connection['postfields']);
    return true;
}



function snow_response(&$response, &$errMsg)
{
    // For debugging of eventual response errors uncomment the following line:
    // hwm_log(HWM_LOG_INF, $response);
    $data = json_decode($response, true);
    if (isset($data['ticket']) && is_string($data['ticket']))
    {
        $param = snow_context_data();
        $process = $param['process'];
        $param = $param['param'];
        if (!snow_set_process_param($process, $param, $data['ticket']))
        {
            $errMsg = hwm_get_last_error();
            return false;
        }
    }
    snow_log('response', $response);
    return true;
}



function snow_log($type, $jsonData)
{
    if (!SNOW_LOG)
        return;
    switch ($type)
    {
    case 'request':
        $msg = 'Sending request to SNOW:';
        break;
    case 'response':
        $msg = 'Received response from SNOW:';
        break;
    default:
        return;
        break;
    }
    error_log(sprintf('[%s] %s', date('Y-m-d H:i:s'), $msg).PHP_EOL, 3,
              SNOW_LOG_FILE);
    $jsonData = json_decode($jsonData, true);
    $jsonData = json_encode($jsonData, JSON_PRETTY_PRINT);
    $jsonData = str_replace("\n", PHP_EOL, $jsonData);
    error_log($jsonData.PHP_EOL, 3, SNOW_LOG_FILE);
}



function snow_context_data($data = null)
{
    static $contextData = array();
    if (isset($data))
        $contextData = $data;
    return $contextData;
}



function snow_get_process_memo($process)
{
    if (!hwm_db_lock(array('client' => false, 'process_memo' => false)))
        return false;
    $memos = hwm_get_process_memo($process);
    if ($memos === false)
    {
        hwm_db_rollback();
        return false;
    }
    hwm_db_rollback();
    if (!is_array($memos))
        return '';
    $memo = '';
    foreach ($memos as $item)
    {
        $ts = hwm_db_ts_adjust($item['timestamp']);
        if ($ts === false)
            $ts = '(no timestamp)';
        else if (strlen($ts) === 14)
            $ts = substr($ts, 0, 4).'-'.substr($ts, 4, 2).'-'.substr($ts, 6, 2)
                  .' '.substr($ts, 8, 2).':'.substr($ts, 10, 2).':'
                  .substr($ts, 12, 2);
        $text = $ts.' '.$item['user'].':'."\n".$item['text']."\n";
        $text = str_replace(array("\r\n", "\n"), array("\n", "\r\n"), $text);
        if ($memo !== '')
            $text .= "\n";
        $memo = $text.$memo;
    }
    return $memo;
}



function snow_set_process_param($process, $param, $val)
{
    if (!hwm_db_lock(array('client' => false, 'param' => true)))
        return false;
    if (!hwm_set_process_param($process, $param, $val))
    {
        hwm_db_rollback();
        return false;
    }
    /*
     * The following code requires ProcMan 5.0.10! Then it can replace the
     * raw commit below.
    if (!hwm_db_commit())
        return false;
     */
    global $hwm_db_connection;
    hwm_log(HWM_LOG_TRC, 'COMMIT: LOG_ID='.hwm_log_id());
    if (db_commit($hwm_db_connection) === false)
    {
        hwm_db_error();
        return false;
    }
    return true;
}



function request1(&$connection, $data, &$errMsg)
{
    // e.g. $connection['postfields'] = array($data['general']['process'],
    //                                        $data['status']);
    return true;
}

function response1(&$response, &$errMsg)
{
    // e.g. $response = json_decode($response, true);
    return true;
}

function request2(&$connection, $data, &$errMsg)
{
    // e.g. $connection['httpheader'] = array(
    //          'data-header: '.base64_encode(json_encode($data)));
    return true;
}

function response2(&$response, &$errMsg)
{
    // e.g. $response = json_decode(base64_decode($response), true);
    return true;
}

function ext_cmd_request(&$connection, $inData, &$errMsg)
{
    $connection['postfields'] = $inData;
    return true;
}

function ext_cmd_response(&$response, &$errMsg)
{
    return true;
}



//
// Example for Windows/Unix script handovers using the Universal Object Module:
//

function script_dispatcher_request(&$connection, $inData, &$errMsg)
{
    $connection['postfields'] = $inData;
    return true;
}

function script_dispatcher_response(&$response, &$errMsg)
{
    return true;
}

function script_writer_request(&$connection, $inData, &$errMsg)
{
    $connection['ssl_verifypeer'] = true;
    $connection['cainfo'] = 'c:/HORIZONT/var/ssl/horizont.crt';
    $system = isset($inData['system']) ? $inData['system'] : '';
    switch (strtoupper($system))
    {
    case 'WIN_1':
        $connection['url'] =
            'https://win_1.my_company.com:15443/hwm_executor.php';
        break;
    case 'WIN_2':
        $connection['url'] =
            'https://win_2.my_company.com:20443/hwm_executor.php';
        break;
    case 'UX_1':
        $connection['url'] =
            'https://ux_1.my_company.com:20443/hwm_executor.php';
        break;
    default:
        break;
    }
    $connection['postfields'] = array(
        'command' => 'script_writer',
        'data' => $inData['data'],
    );
    return true;
}

function script_writer_response(&$response, &$errMsg)
{
    return true;
}



//
// Callbacks for HWM REST API
//

function rest_api_request(&$connection, $inData, &$errMsg)
{
    $connection['postfields'] = $inData;
    return true;
}

function rest_api_response(&$response, &$errMsg)
{
    // For debugging of eventual response errors uncomment the following line:
    // hwm_log(HWM_LOG_INF, $response);
    return true;
}



//
// Callbacks for autoexecution of activities
//

function autoexec_activity_request(&$connection, $inData, &$errMsg)
{
    $jsonOptions = JSON_UNESCAPED_UNICODE | JSON_INVALID_UTF8_IGNORE;
    $connection['postfields'] = json_encode($inData, $jsonOptions);
    return true;
}

function autoexec_activity_response(&$response, &$errMsg)
{
    // For debugging of eventual response errors uncomment the following line:
    // hwm_log(HWM_LOG_INF, $response);
    $response = json_decode($response, true);
    if (!is_array($response))
    {
        $errMsg = 'Invalid result returned by the execution of the automatic '
                  .'activity.'."\n";
        return false;
    }
    return true;
}



//
// Callbacks for the autoexecution handler
//

function autoexec_handler_request(&$connection, $inData, &$errMsg)
{
    $jsonOptions = JSON_UNESCAPED_UNICODE | JSON_INVALID_UTF8_IGNORE;
    $connection['postfields'] = json_encode($inData, $jsonOptions);
    return true;
}

function autoexec_handler_response(&$response, &$errMsg)
{
    // For debugging of eventual response errors uncomment the following line:
    // hwm_log(HWM_LOG_INF, $response);
    $response = json_decode($response, true);
    if (!is_array($response))
    {
        $errMsg = 'Invalid result returned by the execution of the autoexec '
                  .'handler.'."\n";
        return false;
    }
    return true;
}



//
// Callbacks for the file sweeper
//

function file_sweeper_request(&$connection, $inData, &$errMsg)
{
    $connection['postfields'] = json_encode($inData, JSON_UNESCAPED_UNICODE);
    return true;
}

function file_sweeper_response(&$response, &$errMsg)
{
    // For debugging of eventual response errors uncomment the following line:
    // hwm_log(HWM_LOG_INF, $response);
    $response = json_decode($response, true);
    if (!is_array($response))
    {
        $errMsg = 'Invalid result returned by the execution of the file '
                  .'sweeper.'."\n";
        return false;
    }
    return true;
}



//
// Callbacks for the Control-M module
//

require_once('ctm_lib.php');

function ctm_request(&$connection, $inData, &$errMsg)
{
    /* 
     * This way the Control-M API URL can be set depending on the system
     * parameter:
    $system = isset($inData['system']) ? $inData['system'] : '';
    switch (strtoupper($system))
    {
    case 'PROD':
        $connection['url'] =
            'https://ctm_prod.my_company.com:8443/automation-api';
        break;
    case 'TEST':
    default:
        $connection['url'] =
            'https://ctm_test.my_company.com:8443/automation-api';
        break;
    }
     */
    return ctm_universal_interface_request($connection, $inData, $errMsg);
}

function ctm_response(&$response, &$errMsg)
{
    // For debugging of eventual response errors uncomment the following line:
    // hwm_log(HWM_LOG_INF, $response);
    return ctm_universal_interface_response($response, $errMsg);
}
?>
